@@ -26,7 +26,7 @@ from elisa.plugins.database.models import Artist, MusicAlbum, MusicTrack, \
 from elisa.plugins.gstreamer.amp_master import GstMetadataAmpClient
 
 from twisted.python.failure import Failure
-from twisted.internet import task, defer
+from twisted.internet import task, defer, reactor
 
 from elisa.core.component import Component
 
@@ -56,6 +56,11 @@ class DatabaseParser(Component):
     commit_every_files = 50 # number of files to auto commit
     # what ever is reached first will trigger the auto commit
 
+
+    # how long should the parser wait after it processed a file. This delay is
+    # *not* applied to files that are not updated.
+    between_process_delay = 0.1 # value in seconds.
+
     def __init__(self):
         super(DatabaseParser, self).__init__()
         self.store = None
@@ -75,10 +80,17 @@ class DatabaseParser(Component):
             dfr = GstMetadataAmpClient.create({})
             dfr.addCallback(set_metadata)
             return dfr
+
+        def set_process_delay(result):
+            self.between_process_delay = self.config.get('delay', \
+                    self.between_process_delay)
+
+            return result
            
         dfr = super(DatabaseParser, self).initialize()
         dfr.addCallback(self.load_processors)
         dfr.addCallback(create_client)
+        dfr.addCallback(set_process_delay)
         return dfr
 
     def load_processors(result, self):
@@ -135,12 +147,8 @@ class DatabaseParser(Component):
         path = model.uri.path
         self.debug("query for %s" % path)
 
-        def log_error(failure, path, stat):
-            stat.files_failed.append( (path, failure) )
-
         dfr = self.get_or_create(File, 'path', path)
         dfr.addCallback(self.process, model, stat)
-        dfr.addErrback(log_error, path, stat)
 
         return dfr
 
@@ -173,33 +181,43 @@ class DatabaseParser(Component):
         #dfr.addCallback(self.post_process, file, model)
         dfr = self.gst_process(file, model)
         dfr.addCallback(self._commit)
+        dfr.addCallback(self._delay)
+        return dfr
+
+    def _delay(self, result):
+        dfr = defer.Deferred()
+        reactor.callLater(self.between_process_delay, dfr.callback, result)
         return dfr
 
     def _commit(self, result):
         self._commit_counter += 1
         if self._commit_counter >= self.commit_every_files:
-            if self._commit_delay and not self._commit_delay.called:
+            if self._commit_delay and not self._commit_delay.active():
                 self._commit_delay.cancel()
             self._autocommit()
 
-    def _autocommit(self):
-
-        def reset_delayer(result):
+    def _reset_autocommit_delay(self, result):
+        if self._commit_delay and self._commit_delay.active():
+            # still active. reset it
+            self._commit_delay.reset(self.commit_every_seconds)
+        else:
             delay = reactor.callLater(self.commit_every_seconds, self._autocommit)
             self._commit_delay = delay
-            return result
+
+        return result
+
+    def _autocommit(self):
 
         if not self._commit_counter:
             # nothing changed, do not commit
-            reset_delayer(None)
+            self._reset_autocommit_delay(None)
             return
 
         # reset the commit counter
         self._commit_counter = 0
 
         dfr = self.store.commit()
-        dfr.addCallbak(reset_delayer)
-        return dfr
+        dfr.addCallback(self._reset_autocommit_delay)
 
     def pre_process(self, file, model):
         def check_result(result, result_dfr):
@@ -322,7 +340,12 @@ class DatabaseParser(Component):
             except KeyError:
                 value = None
 
-            if value:
+            if not value is None:
+                # when it is not None, we have to convert it into the right type
+                # or storm will complain (raise a TypeError Exception). That
+                # even applies for 'strings' if unicode is required. The only
+                # other value we are allowed to set is the special None-Type
+                # which is the same as the special 'NULL' Value in SQL.
                 value = typ(value)
 
             setattr(model, attr, value)
@@ -402,9 +425,14 @@ class DatabaseParser(Component):
                 continue
 
             date = datetime.datetime(*[int(r) for r in dates])
-            if date:
-                image.shot_time = date
-                break
+            image.shot_time = date
+            break
+
+        if not image.shot_time:
+            # FIXME: ugly fallback to the modification date as the date the
+            # photo was take in order to always allow us to sort them nice
+            stamp = os.stat(image.file_path).st_mtime
+            image.shot_time = datetime.datetime.fromtimestamp(stamp)
 
         album_name = os.path.basename(os.path.split(image.file_path)[0])
 
