@@ -25,6 +25,9 @@ from elisa.extern.storm_wrapper import wrapper
 from storm.locals import Unicode, Int, DateTime, Bool
 
 from elisa.core.media_uri import MediaUri, quote
+from elisa.core.utils import defer
+
+from elisa.plugins.base.models.media import PlayableModel
 
 
 class MusicAlbum(object):
@@ -38,6 +41,38 @@ class MusicAlbum(object):
     cover_uri = Unicode()
     release_date = DateTime()
 
+    def get_tracks(self):
+        dfr = self.tracks.find()
+        dfr.addCallback(lambda rs: rs.all())
+        dfr.addCallback(lambda items: sorted(items, key=lambda a: a.file_path.lower()))
+        return dfr
+
+    def get_artist_name(self):
+        """
+        Retrieve the artist name associated with the first track of the album.
+        If there's no track or no artist is found, empty string is returned.
+        """
+
+        def got_tracks(tracks):
+            def got_artists(artists):
+                if len(artists):
+                    artist_name = artists[0]
+                else:
+                    artist_name = ''
+                return artist_name
+
+            if len(tracks) > 0:
+                dfr = tracks[0].artists.values(Artist.name)
+                dfr.addCallback(got_artists)
+            else:
+                dfr = defer.succeed('')
+            return dfr
+
+        dfr = self.tracks.find()
+        dfr.addCallback(lambda rs: rs.all())
+        dfr.addCallback(got_tracks)
+        return dfr
+
 class PhotoAlbum(object):
     __storm_table__ = "photo_albums"
     name = Unicode(primary=True)
@@ -54,6 +89,11 @@ class Artist(object):
     # eg: http://www.discogs.com/image/A-56260-1189986364.jpeg
     image_uri = Unicode()
 
+    def get_tracks(self):
+        dfr = self.tracks.find()
+        dfr.addCallback(lambda rs: rs.all())
+        dfr.addCallback(lambda items: sorted(items, key=lambda a: a.file_path.lower()))
+        return dfr
 
 class TrackArtist(object):
     """
@@ -84,6 +124,45 @@ class MusicTrack(object):
     # genre
     genre = Unicode()
 
+    def get_album(self):
+        """
+        Return a deferred that will return the album containing the track as an
+        instance of L{elisa.plugins.base.models.audio.AlbumModel}.
+
+        @rtype:  L{elisa.core.utils.defer.Deferred}
+        """
+        return self.album
+
+    def get_artists(self):
+        """
+        Return a deferred that will return the list of artist names for the
+        track as C{unicode} instances.
+
+        @rtype:  L{elisa.core.utils.defer.Deferred}
+        """
+        def values(result):
+            return result.values(Artist.name)
+
+        def set_distinct(result_set):
+            result_set.config(distinct=True)
+            return result_set
+
+        deferred = self.artists.find()
+        deferred.addCallback(set_distinct)
+        deferred.addCallback(values)
+        return deferred
+
+    def get_playable_model(self):
+        """
+        Return a deferred that will return an instance of
+        L{elisa.plugins.base.models.media.PlayableModel} for the track.
+
+        @rtype:  L{elisa.core.utils.defer.Deferred}
+        """
+        model = PlayableModel()
+        model.uri = MediaUri('file://' + self.file_path)
+        model.title = self.title
+        return defer.succeed(model)
 
 class Image(object):
     """
