@@ -21,6 +21,7 @@
 from elisa.core import common
 from elisa.core.media_uri import MediaUri
 from elisa.core.utils.i18n import install_translation
+from elisa.core.utils.cancellable_defer import CancelledError
 
 from elisa.plugins.base.models.media import PlayableModel
 from elisa.plugins.database.models import *
@@ -120,8 +121,11 @@ def music_lib_decades_decorator(controller):
 
 class GenericArtistsController(HierarchyController):
 
+    artist_controller_path = '/poblesec/database/music/albums'
+
     def initialize(self):
-        dfr= super(GenericArtistsController, self).initialize()
+        dfr = super(GenericArtistsController, self).initialize()
+        self.actions.extend(self.make_actions())
         return dfr
 
     def node_clicked(self, widget, item):
@@ -129,15 +133,21 @@ class GenericArtistsController(HierarchyController):
         selected_item = self.nodes._widgets[widget_index]
 
         if selected_item != self._previous_clicked:
+            self.cancel_deferreds(id(self))
             selected_item.activate(previous=self._previous_clicked)
 
             self._previous_clicked = selected_item
 
             if isinstance(item, Action):
-                item.run()
+                def silence_canceller(failure):
+                    failure.trap(CancelledError)
+
+                dfr = item.run()
+                dfr.addErrback(silence_canceller)
+                self.register_deferred(id(self), dfr)
             else:
                 browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                path = '/poblesec/database/music/albums'
+                path = self.artist_controller_path
                 deferred = browser.history.append_controller(path, item.name, artist=item)
 
     def make_actions(self):
@@ -160,9 +170,11 @@ class GenericArtistsController(HierarchyController):
 
 class ArtistsDbController(GenericArtistsController):
 
-    def initialize(self):
+    def initialize(self, artists=None):
         deferred = super(ArtistsDbController, self).initialize()
 
+        self.artists = artists
+
         def got_artists(artists):
             artists = sorted(artists, key=lambda a: a.name.lower())
             self.model.extend(artists)
@@ -175,11 +187,23 @@ class ArtistsDbController(GenericArtistsController):
         def get_artists(self):
             return common.application.store.find(Artist)
 
-        deferred.addCallback(get_artists)
-        deferred.addCallback(sort_artists)
-        deferred.addCallback(got_artists)
+        def populate_model(self, artists):
+            self.model.extend(artists)
+            return self
+
+        if artists is None:
+            deferred.addCallback(get_artists)
+            deferred.addCallback(sort_artists)
+            deferred.addCallback(got_artists)
+        else:
+            deferred.addCallback(populate_model, artists)
         return deferred
 
+    def clean(self):
+        dfr = super(ArtistsDbController, self).clean()
+        self.artists = None
+        return dfr
+
 class ArtistsDbViewMode(ArtistsViewMode):
 
     def get_label(self, item):
@@ -232,9 +256,12 @@ class ArtistsDbListSwitcherController(ListSwitcherController):
 
 class GenericAlbumsDbController(HierarchyController):
 
+    album_controller_path = '/poblesec/database/music/tracks'
+
     def initialize(self, artist=None):
         self.artist = artist
-        dfr= super(GenericAlbumsDbController, self).initialize()
+        dfr = super(GenericAlbumsDbController, self).initialize()
+        self.actions.extend(self.make_actions())
         return dfr
 
     def make_actions(self):
@@ -292,16 +319,19 @@ class GenericAlbumsDbController(HierarchyController):
 
             if isinstance(item, Action):
                 dfr = item.run()
-            elif hasattr(item, 'artist'):
+            elif hasattr(item, 'all_tracks'):
                 # Dummy "All Tracks" album
                 browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
                 dfr = browser.history.append_controller('/poblesec/database/music/tracks',
                                                         item.name,
                                                         artist=item.artist)
-
+            elif hasattr(item, 'action') and issubclass(item.action, Action):
+                action = item.action(self)
+                dfr = action.run(item)
             else:
                 browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                dfr = browser.history.append_controller('/poblesec/database/music/tracks',
+                path = self.album_controller_path
+                dfr = browser.history.append_controller(path,
                                                         item.name,
                                                         album=item)
             return dfr
@@ -309,9 +339,11 @@ class GenericAlbumsDbController(HierarchyController):
 
 class AlbumsDbController(GenericAlbumsDbController):
 
-    def initialize(self, artist=None):
+    def initialize(self, artist=None, albums=None):
         deferred = super(AlbumsDbController, self).initialize(artist=artist)
 
+        self.albums = albums
+
         def sort_albums(result_set):
             result_set.order_by(MusicAlbum.name)
             return result_set.all()
@@ -321,12 +353,25 @@ class AlbumsDbController(GenericAlbumsDbController):
             if self.artist:
                 all_tracks_album = MusicAlbum()
                 all_tracks_album.name = u'All Tracks'
+                # fake attribute to differentiate the 'All tracks' album from
+                # the regular albums
+                all_tracks_album.all_tracks = True
+                # FIXME: very wrong hot plumbing because we need get_tracks()
+                # to return at some other place.
+                all_tracks_album.get_tracks = lambda: defer.succeed([])
                 all_tracks_album.artist = self.artist # dynamic field
                 self.model.append(all_tracks_album)
 
             self.model.extend(albums)
             return self
 
+        def populate_model(self, albums):
+            self.model.extend(albums)
+            return self
+
+        if albums is not None:
+            return deferred.addCallback(populate_model, albums)
+
         if artist is None:
             # Show all albums
             def get_albums(self):
@@ -357,6 +402,11 @@ class AlbumsDbController(GenericAlbumsDbController):
 
         return deferred
 
+    def clean(self):
+        dfr = super(AlbumsDbController, self).clean()
+        self.albums = None
+        return dfr
+
 
 class AlbumsDbViewMode(AlbumsViewMode):
 
@@ -373,7 +423,7 @@ class AlbumsDbViewMode(AlbumsViewMode):
     def get_default_image(self, item):
         if isinstance(item, Action):
             return item.icon
-        elif hasattr(item, 'artist'):
+        elif hasattr(item, 'all_tracks'):
             # "All Tracks" fake album
             return 'elisa.plugins.poblesec.by_track'
         return super(AlbumsDbViewMode, self).get_default_image(item)
@@ -381,7 +431,7 @@ class AlbumsDbViewMode(AlbumsViewMode):
     def get_image(self, item, theme):
         if isinstance(item, Action):
             return None
-        elif hasattr(item, 'artist'):
+        elif hasattr(item, 'all_tracks'):
             # "All Tracks" fake album
             return None
         return super(AlbumsDbViewMode, self).get_image(item, theme)
@@ -389,7 +439,7 @@ class AlbumsDbViewMode(AlbumsViewMode):
     def get_preview_image(self, item, theme):
         if isinstance(item, Action):
             return None
-        elif hasattr(item, 'artist'):
+        elif hasattr(item, 'all_tracks'):
             # "All Tracks" fake album
             return None
         return super(AlbumsDbViewMode, self).get_preview_image(item, theme)
@@ -434,6 +484,7 @@ class GenericTracksDbController(HierarchyController):
 
         dfr= super(GenericTracksDbController, self).initialize()
 
+        self.actions.extend(self.make_actions())
         return dfr
 
     def item_to_label(self, item):
@@ -505,11 +556,14 @@ class GenericTracksDbController(HierarchyController):
             self._previous_clicked = selected_item
             if isinstance(item, Action):
                 item.run()
+            elif hasattr(item, 'action') and issubclass(item.action, Action):
+                action = item.action(self)
+                dfr = action.run(item)
             else:
                 browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
                 path = self.track_controller_path
                 title = item.title
-                tracks = [t for t in self.model if isinstance(t, MusicTrack)]
+                tracks = [t for t in self.model if not isinstance(t, Action)]
                 args = {'track': item,
                         'tracks': tracks}
                 dfr = browser.history.append_controller(path, title, **args)
@@ -526,6 +580,13 @@ class TracksDbController(GenericTracksDbController):
         if track:
             return deferred
 
+        def populate_model(self, tracks):
+            self.model.extend(tracks)
+            return self
+
+        if tracks is not None:
+            return deferred.addCallback(populate_model, tracks)
+
         def got_tracks(tracks):
             self.model.extend(tracks)
             return self
@@ -584,13 +645,6 @@ class DBTracksViewMode(TracksViewMode):
         if isinstance(item, Action):
             return defer.succeed(item.subtitle)
 
-        def values(result):
-            return result.values(Artist.name)
-
-        def set_distinct(result_set):
-            result_set.config(distinct=True)
-            return result_set
-
         def set_values(result_list):
             result_list_len = len(result_list)
             if result_list_len == 0:
@@ -602,11 +656,9 @@ class DBTracksViewMode(TracksViewMode):
                 sublabel = ', '.join(result_list)
             return sublabel
 
-        deferred = item.artists.find()
-        deferred.addCallback(set_distinct)
-        deferred.addCallback(values)
-        deferred.addCallback(set_values)
-        return deferred
+        dfr = item.get_artists()
+        dfr.addCallback(set_values)
+        return dfr
 
     def get_default_image(self, item):
         if isinstance(item, Action):
