@@ -19,7 +19,7 @@
 from elisa.core.utils.i18n import install_translation
 _ = install_translation('database')
 
-from twisted.internet import defer
+from twisted.internet import defer, task
 
 from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
 from elisa.plugins.poblesec.base.hierarchy import HierarchyController
@@ -41,11 +41,12 @@ from elisa.core.media_uri import MediaUri
 from elisa.plugins.poblesec.actions import Action
 from elisa.plugins.poblesec.actions import ViewSlideshowAction
 from elisa.plugins.database.actions import PhotoAddToFavoritesAction
-from elisa.plugins.database.actions import ViewPhotoAlbumSlideshowAction
+from elisa.plugins.database.actions import ViewPhotoAlbumSlideshowAction, \
+        ViewPhotoDateSlideshowAction
 
 from elisa.plugins.favorites.models import FavoritesItem
 
-import os
+import os, datetime
 import os.path
 
 
@@ -76,11 +77,11 @@ def photo_lib_images_decorator(controller):
     return defer.succeed(None)
 
 
-def photo_lib_times_decorator(controller):
+def photo_lib_month_decorator(controller):
     times = Link()
-    times.controller_path = '/poblesec/database/photo/by_time'
-    times.label = _('By time')
-    times.icon = 'elisa.plugins.poblesec.by_genre' # FIXME
+    times.controller_path = '/poblesec/database/photo/by_date'
+    times.label = _('Browse by date')
+    times.icon = 'elisa.plugins.poblesec.by_decade'
     controller.model.append(times)
     return defer.succeed(None)
 
@@ -150,6 +151,7 @@ class GenericPhotosController(HierarchyController):
         self.photos = photos
 
         dfr = super(GenericPhotosController, self).initialize()
+        self.actions.extend(self.make_actions())
         return dfr
 
     def make_actions(self):
@@ -249,6 +251,47 @@ class PhotosController(GenericPhotosController):
 
         return dfr
 
+class PhotosMonthController(GenericPhotosController):
+
+    def initialize(self, month=None, year=None):
+        self.date = datetime.datetime(year, month, 1)
+
+        dfr = super(PhotosMonthController, self).initialize()
+
+        def get_images(result):
+            store = common.application.store
+            try:
+                last = datetime.datetime(year, month+1, 1)
+            except ValueError:
+                # we reached the month 12 + 1, so we have to skip to
+                # january next year
+                last = datetime.datetime(year+1, 1, 1)
+
+            dfr = store.find(Image, Image.shot_time >= self.date,
+                        Image.shot_time < last)
+
+            return dfr
+
+        def got_images(images):
+            self.model.extend(images)
+            return self
+
+        def sort_images(result_set):
+            result_set.order_by(Image.shot_time, Image.file_path)
+            return result_set.all()
+
+        dfr.addCallback(get_images)
+        dfr.addCallback(sort_images)
+        dfr.addCallback(got_images)
+
+        return dfr
+
+    def make_actions(self):
+        action = ViewSlideshowAction(self)
+        action.title = _('Play Slideshow')
+        subtitle = self.date.strftime(_("Slideshow of Photos from %B, %Y"))
+        action.subtitle = subtitle
+        return [action]
 
 class PhotoAlbumViewMode(GenericListViewMode, ThumbnailMixin):
 
@@ -298,6 +341,7 @@ class GenericPhotoAlbumsController(HierarchyController):
 
     def initialize(self):
         dfr = super(GenericPhotoAlbumsController, self).initialize()
+        self.actions.extend(self.make_actions())
         return dfr
 
     def make_actions(self):
@@ -328,6 +372,93 @@ class GenericPhotoAlbumsController(HierarchyController):
                 dfr = history.append_controller('/poblesec/database/photo/list',
                                                         model.name, album=model)
 
+class PhotoTimesViewMode(GenericListViewMode):
+
+    def get_label(self, item):
+        if isinstance(item, Action):
+            return defer.succeed(item.title)
+
+        return defer.succeed(item.strftime(_('%B, %Y')))
+
+    def get_default_image(self, item):
+        if isinstance(item, Action):
+            return item.icon
+        resource = 'elisa.plugins.poblesec.photo_album'
+        return resource
+
+    def get_image(self, item, theme):
+        return None
+
+    def get_preview_image(self, item, theme):
+        if isinstance(item, Action):
+            return None
+        return None
+
+class PhotosTimesController(HierarchyController):
+
+    query = "SELECT DISTINCT STRFTIME('%m %Y', shot_time)" \
+            " from images where shot_time not NULL order by shot_time;"
+
+    def initialize(self):
+        dfr = super(PhotosTimesController, self).initialize()
+
+        def get_times(result):
+            store = common.application.store
+            dfr = store.execute(self.query)
+            return dfr
+
+        def get_all(result):
+            return result.get_all()
+
+        def iterate(rows):
+            for row in rows:
+                month, year = row[0].split()
+                date = datetime.datetime(int(year), int(month), 1)
+                self.model.append(date)
+                yield date
+
+        def got_times(times):
+            return task.coiterate(iterate(times))
+
+        def done(res):
+            return self
+
+        dfr.addCallback(get_times)
+        dfr.addCallback(get_all)
+        dfr.addCallback(got_times)
+        dfr.addCallback(done)
+        return dfr
+
+    def make_actions(self):
+        actions = []
+
+        action = ViewPhotoDateSlideshowAction(self)
+        action.title = _('Play Slideshow')
+        action.sub_title = _('Slideshow of All Photos')
+        actions.append(action)
+
+        return actions
+
+     # copy-paste-code again :(
+    def node_clicked(self, widget, model):
+        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(model))
+        selected_item = self.nodes._widgets[widget_index]
+
+        if selected_item != self._previous_clicked:
+            selected_item.activate(previous=self._previous_clicked)
+
+            self._previous_clicked = selected_item
+
+            if isinstance(model, Action):
+                model.run()
+            else:
+                controllers = self.frontend.retrieve_controllers('/poblesec/browser')
+                history = controllers[0].history
+                dfr = history.append_controller('/poblesec/database/photo/by_month',
+                        model.strftime(_('%B, %Y')), year=model.year, \
+                        month=model.month)
+
+
 
 class PhotoAlbumsController(GenericPhotoAlbumsController):
 
@@ -370,6 +501,38 @@ class PhotoListSwitcherController(ListSwitcherController):
              PhotoGrid]
     default_mode = PhotoGrid
 
+# Photo by month
+class PhotoMonthVerticalWithPreview(PhotosMonthController, MenuItemPreviewListController):
+    view_mode = PhotoViewMode
+
+class PhotoMonthCoverflow(PhotosMonthController, ImageWithReflectionCoverflowController):
+    view_mode = PhotoViewMode
+
+class PhotoMonthGrid(PhotosMonthController, GridItemGridController):
+    view_mode = PhotoViewMode
+
+class PhotoMonthListSwitcherController(ListSwitcherController):
+    modes = [PhotoMonthVerticalWithPreview,
+             PhotoMonthCoverflow,
+             PhotoMonthGrid]
+    default_mode = PhotoMonthGrid
+
+# Photo Times Overview
+class PhotoTimesVerticalWithPreview(PhotosTimesController, MenuItemPreviewListController):
+    view_mode = PhotoTimesViewMode
+
+class PhotoTimesCoverflow(PhotosTimesController, ImageWithReflectionCoverflowController):
+    view_mode = PhotoTimesViewMode
+
+class PhotoTimesGrid(PhotosTimesController, GridItemGridController):
+    view_mode = PhotoTimesViewMode
+
+class PhotoTimesListSwitcherController(ListSwitcherController):
+    modes = [PhotoTimesVerticalWithPreview,
+             PhotoTimesCoverflow,
+             PhotoTimesGrid]
+    default_mode = PhotoTimesVerticalWithPreview
+
 # Albums
 class AlbumsVerticalWithPreview(PhotoAlbumsController, MenuItemPreviewListController):
     view_mode = PhotoAlbumViewMode
