@@ -88,12 +88,15 @@ def music_playlists_decorator(controller):
 
     return defer.succeed(None)
 
+# FIXME: even though the by_artist Icon is mainly used for the music section we
+# want to have persons silhouette. Later we should replace it but for now we use
+# the same one
 def recently_added_photos_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/photo/photos_added'
     link.label = _('Recently added Photos')
     link.controller_args = {'limit' : 20}
-    link.icon = 'elisa.plugins.poblesec.recently_added_photos'
+    link.icon = 'elisa.plugins.poblesec.by_artist'
     controller.model.append(link)
 
     return defer.succeed(None)
@@ -103,19 +106,17 @@ def last_played_photos_decorator(controller):
     link.controller_path = '/poblesec/database/photo/photos_played'
     link.label = _('Recently viewed Photos')
     link.controller_args = {'limit':20,  'order_by' : 'last_played'}
-    link.icon = 'elisa.plugins.poblesec.recently_viewed_photos'
+    link.icon = 'elisa.plugins.poblesec.by_artist'
     controller.model.append(link)
 
     return defer.succeed(None)
 
 def most_played_photos_decorator(controller):
-    # disabled for now
-    return defer.succeed(None)
     link = Link()
     link.controller_path = '/poblesec/database/photo/photos_played'
-    link.label = _('Most watched Photos')
+    link.label = _('Most viewed Photos')
     link.controller_args = {'limit': 20,}
-    link.icon = 'elisa.plugins.poblesec.playlist'
+    link.icon = 'elisa.plugins.poblesec.by_artist'
     controller.model.append(link)
 
     return defer.succeed(None)
@@ -123,8 +124,8 @@ def most_played_photos_decorator(controller):
 def pictures_playlists_decorator(controller):
     link = Link()
     link.controller_path = '/poblesec/database/pictures/playlists'
-    link.label = _('Playlists')
-    link.icon = 'elisa.plugins.poblesec.playlist'
+    link.label = _('Your History')
+    link.icon = 'elisa.plugins.poblesec.by_artist'
     controller.model.append(link)
 
     return defer.succeed(None)
