@@ -45,43 +45,108 @@ class ScannerWidget(Widget):
         self.image.visible = True
         self.image.x = 0.85
         self.image.y = 0.0
-        self.image.width = 0.15
+        self.image.width = 0.189
         self.image.height = 1
+
         self.text = Text()
         self.add(self.text)
         self.text.bg_a = 0
         self.text.font_family = "Liberation Sans"
         self.text.font_height = 1
+        self.text.font_weight = pgm.TEXT_WEIGHT_BOLD
         self.text.alignment = pgm.TEXT_ALIGN_RIGHT
-        self.text.y = 0.25
+        self.text.y = 0.0
         self.text.width = 0.80
         self.text.height = 0.5
         self.text.visible = True
-        self.text.fg_color = (102, 102, 102, 255)
+        self.text.fg_color = (130, 130, 130, 255)
+
+        self.subtext = Text()
+        self.add(self.subtext)
+        self.subtext.bg_a = 0
+        self.subtext.font_family = "Liberation Sans"
+        self.subtext.font_height = 1
+        self.subtext.font_weight = pgm.TEXT_WEIGHT_LIGHT
+        self.subtext.alignment = pgm.TEXT_ALIGN_RIGHT
+        self.subtext.y = 0.5
+        self.subtext.width = 0.80
+        self.subtext.height = 0.42
+        self.subtext.visible = True
+        self.subtext.fg_color = (102, 102, 102, 255)
        
         # create animation stuff
+        self._loop_call = None
         self.animated = implicit.AnimatedObject(self)
         self._rotation_matrix = pgm.mat4x4_new_identity()
         self._rotation_matrix.translate(0.5, 0.5, 0.0)
         self._rotation_matrix.rotate_z(-math.pi / 30.0)
         self._rotation_matrix.translate(-0.5, -0.5, 0.0)
+        
+        self._stopping_animation = False
+
+        # to be able to be more verbose on statistics
+        self._currently_scanning = None
+        self._percent = None
 
     def running__set(self, running):
         if running:
             self.opacity = 255
-            # possible bug: running = false and running = true could lead to
-            # having the loop running twice
-            self._transform_mapping_matrix_cb()
+            if not self._loop_call:
+                self._transform_mapping_matrix_cb()
         else:
             self.opacity = 0
 
     running = property(fset=running__set)
 
+    def currently_scanning__set(self, scan_resource):
+        if self._currently_scanning:
+            # we have to unbind from the old one
+            self.files_scanned = 0
+            self._currently_scanning.unbind_object(self)
+            self._currently_scanning = None
+            self.text.label = ''
+            self.subtext.label = ''
+
+        if scan_resource is None:
+            # scanning stopped
+            self.running = False
+        else:
+            self.running = True
+            self._currently_scanning = scan_resource
+
+            self.text.label = _('Scanning %s') % scan_resource.name
+            self.subtext.label = _('estimating')
+            scan_resource.bind('files_scanned', self, 'files_scanned')
+
+    currently_scanning = property(fset=currently_scanning__set)
+
+    def files_scanned__set(self, counter):
+        if not self._currently_scanning.files_total:
+            # there seem to be no total amount. we can't calculate
+            return
+
+        percent = int(counter * 100. / self._currently_scanning.files_total)
+
+        if self._percent == percent:
+            # only update if the percent changed
+            return
+
+        self._percent = percent
+        self.subtext.label = _('%d%% Complete') % self._percent
+
+    files_scanned = property(fset=files_scanned__set)
+    
+    def clean(self):
+        super(ScannerWidget, self).clean()
+        self._stopping_animation = True
+
     def _transform_mapping_matrix_cb(self):
-        if self.opacity and self.visible:
+        self._loop_call = None
+        if self.opacity and self.visible and not self._stopping_animation:
             # only do the animation as long as we are shown
             self.image.mapping_matrix *= self._rotation_matrix
-            reactor.callLater(0.017, self._transform_mapping_matrix_cb)
+            self._loop_call = reactor.callLater(0.017, \
+                    self._transform_mapping_matrix_cb)
 
 def scanning_decorator(controller):
     uri = MediaUri("media_scanner://localhost/statistic")
@@ -100,13 +165,12 @@ def scanning_decorator(controller):
     widget.opacity = 0
     widget.visible = True
     widget.image.set_from_file(wait_icon)
-    widget.width = 0.20
-    widget.height = 0.075
-    widget.x = 0.78
-    widget.y = 0.925
-    widget.text.label = _('Scanning Media')
+    widget.width = 0.35
+    widget.height = 0.08
+    widget.x = 0.62
+    widget.y = 0.9
 
-    model.bind('running', widget, 'running')
+    model.bind('currently_scanning', widget, 'currently_scanning')
 
     return defer.succeed(widget)
 
