@@ -43,8 +43,12 @@ class DBMixin(object):
         self.store = store.DeferredStore(self.db)
         dfr = self.store.start()
         dfr.addCallback(lambda x: self._create_tables())
+        dfr.addCallback(self._set_store_to_application)
         return dfr
 
+    def _set_store_to_application(self, result=None):
+        common.application.store = self.store
+    
     def tearDown(self):
         self.unpatch_application()
         return self.store.stop()
