@@ -16,9 +16,12 @@
 
 from elisa.core import common
 from elisa.core.media_uri import MediaUri
+from elisa.core.utils import defer
 
 from elisa.plugins.base.models.audio import TrackModel
 from elisa.plugins.base.models.media import PlayableModel
+from elisa.plugins.base.models.device import VolumeModel
+from elisa.plugins.base.messages.device import NewDeviceDetected
 
 from elisa.plugins.poblesec.base.hierarchy import HierarchyController
 from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
@@ -31,9 +34,58 @@ from elisa.plugins.poblesec.base.preview_list import \
 from elisa.plugins.poblesec.base.coverflow import \
     ImageWithReflectionCoverflowController
 from elisa.plugins.poblesec.base.grid import GridItemGridController
+from elisa.plugins.poblesec.link import DeviceLink
 
 from twisted.internet import task
 
+def ipod_decorator(controller):
+    """
+    Adds an iPod entry to the music menu if there is an ipod plugge in.
+    """
+    uri = MediaUri("volumes://?filter=ipod")
+    model, dfr = common.application.resource_manager.get(uri, None)
+
+    def add_link(model):
+        """
+        Adds a link to the controller for the iPod described in C{model}.
+        """
+        link = DeviceLink(model.udi)
+        link.controller_path = "/poblesec/music/ipod"
+        link.icon = "elisa.plugins.poblesec.ipod"
+        uri = MediaUri({'scheme': 'ipod',
+                        'host': '',
+                        'path': model.mount_point})
+        uri.set_param('action', 'tracks')
+        uri.set_param('search', 'artist')
+        link.controller_args = {'uri': uri}
+        link.label = model.name
+        controller.model.append(link)
+
+
+    def do_list_hardware(model):
+        devices = sorted(model.devices, key=lambda i: i.name.lower())
+
+        for dev in devices:
+            if dev.mount_point != None:
+                add_link(dev)
+
+    dfr.addCallback(do_list_hardware)
+
+    def on_device_added(message, sender):
+        if message.model.protocol == 'ipod' \
+          and message.model.mount_point != None:
+            add_link(message.model)
+
+    bus = common.application.bus
+    bus.register(on_device_added, NewDeviceDetected)
+
+    def on_clean(controller):
+        bus.unregister(on_device_added)
+        return defer.succeed(None)
+
+    controller.connect('clean', on_clean)
+
+    return dfr
 
 class ArtistsIpodController(HierarchyController):
 
