@@ -25,5 +25,6 @@ setup(name='elisa-plugin-ipod',
                           'elisa.plugins.ipod.controller:AlbumsIpodListSwitcherController'),
                          ('/poblesec/music/ipod/tracks',
                           'elisa.plugins.ipod.controller:TracksIpodListSwitcherController'),],
-    decorator_mappings=[],
+    decorator_mappings=[('/poblesec/music_menu',
+                         'elisa.plugins.ipod.controller:ipod_decorator'),],
     cmdclass=cmdclass)
