@@ -51,6 +51,7 @@ class PigmentController(Controller, gobject.GObject):
         self.widget.visible = True
         self.widget_animated = implicit.AnimatedObject(self.widget)
         self.widget_animated.mode = implicit.REPLACE
+        self.pending_decorator_deferreds = []
 
     def set_frontend(self, frontend):
         """
@@ -94,8 +95,10 @@ class PigmentController(Controller, gobject.GObject):
         pass
 
     def clean(self):
-        dfr = DeferredList(self.emit('clean'))
-
+        dfr = DeferredList(self.emit('clean') + 
+                self.pending_decorator_deferreds)
+        self.widget.clean()
+        
         def call_parent(ignored):
             return super(PigmentController, self).clean()
 
