@@ -35,6 +35,13 @@ import pkg_resources
 from elisa.plugins.pigment.pigment_input import PigmentInput
 import weakref
 import threading
+import sys
+
+try:
+    import dbus
+    from elisa.plugins.pigment.dbus_frontend import DBusFrontend
+except ImportError:
+    pass
 
 class ControllerNotFound(Exception):
     pass
@@ -51,6 +58,7 @@ class PigmentFrontendExtensionMixin(Loggable):
 
         self._controller_store = {}
 
+
     def _load_controller_mappings(self):
         """
         Load the controller mappings from all plugins' metadata.
@@ -86,7 +94,8 @@ class PigmentFrontendExtensionMixin(Loggable):
         """
         self.controller_matcher.add_pattern(path_pattern, controller)
 
-    def create_controller(self, path, config=None, **kwargs):
+    def create_controller(self, path, config=None,
+            wait_for_decorators=False, **kwargs):
         """
         Create a controller for the given path.
 
@@ -111,7 +120,7 @@ class PigmentFrontendExtensionMixin(Loggable):
         dfr.addCallback(self._store_controller, path)
         dfr.addCallback(self._set_frontend)
         dfr.addCallback(self._set_path, path)
-        dfr.addCallback(self._decorate)
+        dfr.addCallback(self._decorate, wait_for_decorators)
         return dfr
 
     def _store_controller(self, controller, path):
@@ -176,7 +185,8 @@ class PigmentFrontendExtensionMixin(Loggable):
         # swallow the failure
         return None
 
-    def _iterate_decorators_callback(self, iterator, results, controller):
+    def _iterate_decorators_callback(self, iterator, results, controller, dfr):
+        controller.pending_decorator_deferreds.remove(dfr) 
         return controller
 
     def _iterate_decorators(self, controller, matches, resultlist):
@@ -205,7 +215,7 @@ class PigmentFrontendExtensionMixin(Loggable):
             dfr.addErrback(self._decorator_errback, resultlist)
             yield dfr
 
-    def _decorate(self, controller):
+    def _decorate(self, controller, wait_for_decorators):
         try:
             matches = self.decorator_matcher.match(controller.path, all=True)
         except MatchNotFound:
@@ -214,9 +224,13 @@ class PigmentFrontendExtensionMixin(Loggable):
 
         results = []
         dfr = task.coiterate(self._iterate_decorators(controller, matches, results))
-        dfr.addCallback(self._iterate_decorators_callback, results, controller)
+        controller.pending_decorator_deferreds.append(dfr)
+        dfr.addCallback(self._iterate_decorators_callback, results, controller, dfr)
 
-        return dfr
+        if wait_for_decorators:
+            return dfr
+
+        return defer.succeed(controller)
 
 class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
     """
@@ -244,6 +258,7 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
     default_config = {'window_width': '0',
                       'touchscreen': '0',
                       'use_gtk': '0',
+                      'headless': '0',
                       'start_fullscreen': '1',
                       'controller_path': '',
                       'theme': None,
@@ -257,6 +272,7 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
                   'use_gtk'     : 'Start the frontend inside a gtk window',
                   'touchscreen' : 'If set to 1, the mouse behaviour will be'
                                   ' adapted for touchscreen equipped hardware.',
+                  'headless': 'Start in headless mode',
                   'start_fullscreen': 'If set to 1, Elisa will start fullscreen'
                                       ' otherwise windowed',
                   'controller_path': 'Path used to decide which controller'
@@ -338,6 +354,9 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
 
         fullscreen = self.config.as_bool('start_fullscreen')
 
+        headless = '--headless' in sys.argv or \
+            self.config.as_bool('headless')
+
         if self.config.as_bool('use_gtk') is True:
             self.debug("using gtk")
 
@@ -352,7 +371,8 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
             self.gtk_window.set_focus_child(embed)
             self.gtk_window.set_title('Elisa Media Center')
 
-            self.gtk_window.show_all()
+            if not headless:
+                self.gtk_window.show_all()
 
             if fullscreen:
                 self.gtk_window.fullscreen()
@@ -360,7 +380,8 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
                 self.gtk_window.unfullscreen()
         else:
             viewport.fullscreen = fullscreen
-            viewport.show()
+            if not headless:
+                viewport.show()
 
             # Pigment configure events are sent on window move/resize. This
             # callback allows to regenerate our texts considering the new
@@ -419,6 +440,8 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         common.application.input_manager.register_component(self.pigment_input)
         common.application.input_manager.connect('input-event', self.handle_input)
 
+        self._initialize_dbus()
+
         return dfr
     
     def _viewport_delete_event(self, viewport, event):
@@ -520,6 +543,8 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         image.set_from_image(master)
 
     def clean(self):
+        self._clean_dbus()
+
         if self.controller:
              self.controller.removed()
 
@@ -565,3 +590,29 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
 
     def _hide_cursor(self):
         self.viewport.cursor = pgm.VIEWPORT_NONE
+
+    def _initialize_dbus(self):
+        if 'dbus' not in sys.modules:
+            # no dbus support
+            return
+
+        bus = dbus.SessionBus()
+        self.bus_name = \
+                dbus.service.BusName('com.fluendo.Elisa.Plugins.Pigment', bus)
+    
+        self.dbus_frontend = DBusFrontend(self, bus, 
+                '/com/fluendo/Elisa/Plugins/Pigment/Frontend', self.bus_name)
+
+    def _clean_dbus(self):
+        if 'dbus' not in sys.modules:
+            # no dbus support
+            return
+
+        bus = dbus.SessionBus()
+        self.dbus_frontend.remove_from_connection(bus,
+                '/com/fluendo/Elisa/Plugins/Pigment/Frontend')
+        # BusName implements __del__, eew
+        del self.bus_name
+
+        # remove the reference cycle
+        del self.dbus_frontend
