@@ -1,5 +1,5 @@
 from twisted.trial.unittest import TestCase
-from twisted.internet import defer
+from twisted.internet import defer, reactor
 from twisted.python import reflect
 
 from elisa.core.config import Config
@@ -112,7 +112,8 @@ class TestPigmentFrontendExtensionMixin(TestCase):
         def create_controller_callback(controller):
             self.failUnlessEqual(controller.decorated, 1)
 
-        dfr = self.frontend.create_controller('/elisa/album/', Config())
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=True)
         dfr.addCallback(create_controller_callback)
 
         return dfr
@@ -129,7 +130,8 @@ class TestPigmentFrontendExtensionMixin(TestCase):
         def create_controller_callback(controller):
             self.failUnlessEqual(controller.decorated, 1)
 
-        dfr = self.frontend.create_controller('/elisa/album/', Config())
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=True)
         dfr.addCallback(create_controller_callback)
 
         return dfr
@@ -146,7 +148,8 @@ class TestPigmentFrontendExtensionMixin(TestCase):
         def create_controller_callback(controller):
             self.failUnlessEqual(controller.decorated, 0)
 
-        dfr = self.frontend.create_controller('/elisa/album/', Config())
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=True)
         dfr.addCallback(create_controller_callback)
 
         return dfr
@@ -163,7 +166,8 @@ class TestPigmentFrontendExtensionMixin(TestCase):
         def create_controller_callback(controller):
             self.failUnlessEqual(controller.decorated, 3)
 
-        dfr = self.frontend.create_controller('/elisa/album/', Config())
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=True)
         dfr.addCallback(create_controller_callback)
 
         return dfr
@@ -184,7 +188,8 @@ class TestPigmentFrontendExtensionMixin(TestCase):
         def create_controller_callback(controller):
             self.failUnlessEqual(controller.decorated, 3)
 
-        dfr = self.frontend.create_controller('/elisa/album/', Config())
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=True)
         dfr.addCallback(create_controller_callback)
 
         return dfr
@@ -208,7 +213,8 @@ class TestPigmentFrontendExtensionMixin(TestCase):
         def check_clean(controller):
             self.failUnlessEqual(controller.decorated, 0)
             
-        dfr = self.frontend.create_controller('/elisa/album/', Config())
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=True)
         dfr.addCallback(create_controller_callback)
         dfr.addCallback(clean_controller)
         dfr.addCallback(check_clean)
@@ -236,7 +242,8 @@ class TestPigmentFrontendExtensionMixin(TestCase):
         def check_clean(controller):
             self.failUnlessEqual(controller.decorated, 0)
 
-        dfr = self.frontend.create_controller('/elisa/album/', Config())
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=True)
         dfr.addCallback(create_controller_callback)
         dfr.addCallback(clean_controller)
         dfr.addCallback(check_clean)
@@ -317,6 +324,51 @@ class TestPigmentFrontendExtensionMixin(TestCase):
 
         return dfr
 
+    def test_create_no_wait_decorators(self):
+        """
+        Test that create_controller() doesn't wait for all the decorators to be
+        finished before finishing.
+        """
+        decorator_wait_dfr = defer.Deferred()
+        def slow_decorator(controller):
+            return decorator_wait_dfr
+
+        def create_controller_cb(controller):
+            decorator_wait_dfr.callback(controller)
+
+        self.frontend.add_controller('/elisa/album/.*', GenericAlbumController)
+        self.frontend.add_decorator('/elisa/album/.*', slow_decorator)
+
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=False)
+        dfr.addCallback(create_controller_cb)
+
+        return dfr
+    
+    def test_controller_clean_wait_for_decorators(self):
+        """
+        Test that after a create_controller(..., wait_for_decorators=False)
+        controller.clean() does wait for the decorators to finish so that the
+        reactor is not left unclean.
+        """
+        decorator_wait_dfr = defer.Deferred()
+        reactor.callLater(0.1, decorator_wait_dfr.callback, None)
+
+        def slow_decorator(controller):
+            return decorator_wait_dfr
+
+        def create_controller_cb(controller):
+            return controller.clean()
+
+        self.frontend.add_controller('/elisa/album/.*', GenericAlbumController)
+        self.frontend.add_decorator('/elisa/album/.*', slow_decorator)
+
+        dfr = self.frontend.create_controller('/elisa/album/', Config(),
+                wait_for_decorators=False)
+        dfr.addCallback(create_controller_cb)
+
+        return dfr
+        
 
 class PigmentFrontendExtensionMixinStub(PigmentFrontendExtensionMixin):
 
