@@ -20,10 +20,13 @@
 from elisa.core import common
 
 from elisa.core.media_uri import MediaUri
+from elisa.core.utils import defer
+from elisa.core.utils.cancellable_defer import cancellable_deferred_iterator, \
+        CancellableDeferred
 from elisa.plugins.base.models.media import PlayableModel
 from elisa.plugins.base.models.image import ImageModel
 
-from twisted.internet import defer
+from twisted.internet import task
 
 from random import shuffle
 import datetime
@@ -56,37 +59,33 @@ class PlayAllAction(Action):
 
     icon = 'elisa.plugins.poblesec.play_all'
 
-    def playable_models_from_items(self, items):
-        models = []
-        for item in items:
-            playable_model = PlayableModel()
-            playable_model.uri = MediaUri('file://' + item.file_path)
-            models.append(playable_model)
+    @cancellable_deferred_iterator
+    def _build_playlist(self, tracks):
+        for item in self.controller.model:
+            if isinstance(item, Action):
+                continue
 
-        return models
+            if hasattr(item, 'get_playable_model'):
+                dfr = defer.succeed([item])
+            else:
+                dfr = item.get_tracks()
 
-    def get_items(self, item):
-        raise NotImplementedError()
+            dfr.addCallback(lambda result: tracks.extend(result))
+            yield dfr
 
     def run(self):
-        items = [item for item in self.controller.model if not isinstance(item, Action)]
         tracks = []
-        dfrs = []
-        for item in items:
-            dfr = self.get_items(item)
-            dfr.addCallback(self.playable_models_from_items)
-            dfr.addCallback(lambda playable_items: tracks.extend(playable_items))
-            dfrs.append(dfr)
+        cancellable_iterator = self._build_playlist(tracks)
+        def canceller(dfr):
+            cancellable_iterator.cancel()
+        cancellable = CancellableDeferred(canceller)
 
-        if len(dfrs) == 0:
-            self.controller.stop_loading_animation()
-            return defer.succeed(None)
+        dfr = task.coiterate(iter(cancellable_iterator))
+        dfr.addCallback(lambda result: self.order_tracks(tracks))
+        dfr.addCallback(self.play)
 
-        dfr_list = defer.DeferredList(dfrs)
-        dfr_list.addCallback(lambda result: self.order_tracks(tracks))
-        dfr_list.addCallback(self.play)
-
-        return dfr_list
+        dfr.chainDeferred(cancellable)
+        return cancellable
 
     def order_tracks(self, tracks):
         return tracks
@@ -96,15 +95,17 @@ class PlayAllAction(Action):
         # of PlayableModel's before actually starting to play the first item.
         # The playback of the first item should start first and then the list
         # of following items to enqueue in the playlist be built.
-        if items is not None and len(items) > 0:
-            poblesec = self.controller.frontend.retrieve_controllers('/poblesec')[0]
-            player = poblesec.music_player.player
-            poblesec.show_music_player()
-            self.controller.stop_loading_animation()
+        if not items:
+            return items
 
-            player.play_model(items[0])
-            for item in items[1:]:
-                player.enqueue_to_playlist(item)
+        poblesec = self.controller.frontend.retrieve_controllers('/poblesec')[0]
+        player = poblesec.music_player.player
+        poblesec.show_music_player()
+        self.controller.stop_loading_animation()
+
+        player.play_model(items[0])
+        for item in items[1:]:
+            player.enqueue_to_playlist(item)
 
 
 class ShuffleAction(PlayAllAction):
@@ -186,25 +187,16 @@ class AddToFavoritesAction(Action):
             self.title = self.untoggled_title
             self.subtitle = self.untoggled_subtitle
 
-        # actually refresh the list item. This is for the list controllers
+        # visually refresh the list item
+        # FIXME: uses reassignment to inform the widget that some values have
+        # changed
         try:
-            index = self.controller.model.index(self)
-            self.controller.model[index] = self
-            return # FIXME: ugly return in the a try
-        except ValueError:
+            index = self.controller.actions.index(self)
+            self.controller.actions[index] = self
+        except ValueError, exception:
+            # the action is not yet in the list of actions
             pass
 
-        # actually refresh the list item. This is for the grid and coverflow
-        # controllers
-        try:
-            index = self.controller.actions_list.model.index(self)
-            self.controller.actions_list.model[index] = self
-            return # FIXME: ugly return in the a try
-        except ValueError:
-            pass
-        except AttributeError, e:
-            self.controller.warning(e.message)
-
 
 class AddToPlaylistAction(Action):
 
@@ -231,6 +223,43 @@ class LinkAction(Action):
         return dfr
 
 
+class GoToNode(object):
+
+    def __init__(self):
+        self.path = ''
+        self.title = ''
+        self.args = {}
+
+
+class ActionNodesNotSetError(Exception):
+    pass
+
+
+class GoToAction(Action):
+
+    def __init__(self, controller):
+        super(GoToAction, self).__init__(controller)
+
+        self.label = ''
+        self.nodes = []
+
+    def run(self):
+        if not self.nodes:
+            raise ActionNodesNotSetError()
+
+        browser = self.controller.frontend.retrieve_controllers('/poblesec/browser')[0]
+        browser.history.clear()
+
+        def _append_controller(result, *args, **kwargs):
+            return browser.history.append_controller(*args, **kwargs)
+
+        dfr = defer.succeed(True)
+        for node in self.nodes:
+            dfr.addCallback(_append_controller, node.path, node.title, **node.args)
+
+        return dfr
+
+
 class ViewSlideshowAction(Action):
 
     icon = 'elisa.plugins.poblesec.play_all'
