@@ -22,8 +22,6 @@ from elisa.plugins.pigment.widgets.widget import Widget
 from elisa.plugins.pigment.widgets.grid_horizontal import GridHorizontal
 from elisa.plugins.pigment.widgets.list_horizontal import ListHorizontal
 
-from elisa.plugins.pigment.widgets.notifying_list import List as NotifyingList
-
 from elisa.plugins.poblesec.widgets.menu_item import MenuItemWidget
 
 from elisa.plugins.poblesec.widgets.loadable_mixin import LoadableMixin
@@ -201,6 +199,14 @@ class GridController(ListController):
         else:
             return super(GridController, self).handle_input(manager, input_event)
 
+class CustomListHorizontal(ListHorizontal):
+    def _selected_to_range_start(self, selected):
+        half_size = (self.visible_range_size-1.0)/2.0
+        if selected <= half_size or len(self.model) <= self.visible_range_size:
+            visible_range_start = 0.0
+        else:
+            visible_range_start = selected-half_size
+        return visible_range_start
 
 class GridItemGridController(GridController):
 
@@ -270,26 +276,20 @@ class GridItemGridController(GridController):
     def set_frontend(self, frontend):
         super(GridItemGridController, self).set_frontend(frontend)
 
-        n_actions = len(self.actions) or 1
-        self.actions_list = ListHorizontal(MenuItemWidget, n_actions)
+        # visible items set to 3.6 in order to fit a little bit more than 3
+        # items thus making the ones after easily discoverable
+        self.actions_list = CustomListHorizontal(MenuItemWidget, 3.6)
         self.widget.add(self.actions_list)
         # adjust the widget's size: easy way to have nicer look
-        height = 0.08
-        if n_actions == 1:
-            width = 0.4
-        elif n_actions == 2:
-            width = 0.8
-        else:
-            width = 0.9
-        self.actions_list.size = (width, height)
-        self.actions_list.position = ((1.0 - width) / 2, 0.8, 0.0)
+        self.actions_list.size = (1.0, 0.08)
+        self.actions_list.position = (0.05, 0.8, 0.0)
 
         def renderer(item, widget):
             self.frontend.load_from_theme(item.icon, widget.icon)
             widget.label.label = item.title
 
         self.actions_list.set_renderer(renderer)
-        self.actions_list.set_model(NotifyingList(self.actions))
+        self.actions_list.set_model(self.actions)
         self.actions_list.set_selector(self._create_selector())
 
         self.actions_list.connect('item-clicked', self._action_clicked)
