@@ -17,6 +17,7 @@
 # Author: Olivier Tilloy <olivier@fluendo.com>
 
 from elisa.core.component import Component
+from elisa.plugins.pigment.widgets import notifying_list
 
 from twisted.internet import task
 
@@ -46,13 +47,14 @@ class HierarchyController(Component):
     @ivar deferreds: a dictionary of currently pending deferred calls
     @type deferreds: C{dict} of C{list} of
                      {elisa.core.utils.cancellable_defer.CancellableDeferred}
-    @ivar actions: a list of contextual actions for this controller.
-    @type actions: C{list} of L{elisa.plugins.poblesec.actions.Action}
+    @ivar actions:   a list of contextual actions for this controller.
+    @type actions:   L{elisa.plugins.pigment.widgets.notifying_list.List} of
+                     L{elisa.plugins.poblesec.actions.Action}
     """
 
     def initialize(self):
         self.deferreds = {}
-        self.actions = self.make_actions()
+        self.actions = notifying_list.List()
 
         # This call to super has to be done after the initialization of
         # self.actions, because it will take us into the hierarchy of the other
@@ -75,26 +77,12 @@ class HierarchyController(Component):
         """
         raise NotImplementedError()
 
-    def make_actions(self):
-        """
-        Build contextual actions for this controller.
-
-        @return: the actions for the controller
-        @rtype:  C{list} of L{elisa.plugins.poblesec.actions.Action}
-        """
-        return []
-
     def register_deferred(self, key, deferred):
         """
         Register a deferred call to be associated to a given key.
         """
-        # TODO: add a callback/errback to the deferred that removes it from the
-        # dictionary once fired or errbacked?
-        try:
-            self.deferreds[key].append(deferred)
-        except KeyError:
-            self.deferreds[key] = [deferred]
-
+        self.deferreds.setdefault(key, []).append(deferred)
+        
     def cancel_deferreds(self, key):
         """
         Cancel all the currently pending deferred calls associated to one given
