@@ -336,11 +336,45 @@ class MenuItemPreviewListController(PreviewListController):
             self.model[0:0] = self.actions
             return self
 
+        def connect_actions_to_model(self):
+            notifier = self.actions.notifier
+            self._actions_inserted_id = notifier.connect('items-inserted',
+                                                         self._actions_inserted)
+            self._actions_deleted_id = notifier.connect('items-deleted',
+                                                        self._actions_deleted)
+            self._actions_changed_id = notifier.connect('items-changed',
+                                                        self._actions_changed)
+            self._actions_reordered_id = notifier.connect('items-reordered',
+                                                          self._actions_reordered)
+            return self
+
         init_deferred = super(MenuItemPreviewListController, self).initialize(**kwargs)
         init_deferred.addCallback(create_view_mode)
         init_deferred.addCallback(insert_actions)
+        init_deferred.addCallback(connect_actions_to_model)
         return init_deferred
 
+    def clean(self):
+        self.actions.notifier.disconnect(self._actions_inserted_id)
+        self.actions.notifier.disconnect(self._actions_deleted_id)
+        self.actions.notifier.disconnect(self._actions_changed_id)
+        self.actions.notifier.disconnect(self._actions_reordered_id)
+        dfr = super(MenuItemPreviewListController, self).clean()
+        return dfr
+
+    def _actions_inserted(self, notifier, index, actions):
+        self.model[index:index] = actions
+
+    def _actions_deleted(self, notifier, index, actions):
+        self.model[index:index+len(actions)] = []
+
+    def _actions_changed(self, notifier, index, actions):
+        for i, action in enumerate(actions):
+            self.model[index+i] = action
+
+    def _actions_reordered(self, notifier, index, actions):
+        self.model.notifier.emit('items-reordered')
+
     def node_renderer(self, item, widget):
         """
         Render a node using the common API methods defined by the
