@@ -19,6 +19,12 @@ from elisa.core.media_uri import MediaUri
 from elisa.core.common import application
 from elisa.core.utils.i18n import install_translation
 
+from elisa.plugins.base.models.file import DirectoryModel
+from elisa.plugins.base.models.device import VolumeModel
+
+from elisa.plugins.base.messages.device import NewDeviceDetected, \
+                                               DeviceRemoved
+
 from elisa.plugins.pigment.widgets import notifying_list
 
 from elisa.plugins.poblesec.filesystem import FilesystemPreviewListController
@@ -34,6 +40,8 @@ import os.path
 
 _ = install_translation('poblesec')
 
+HOME_URI = MediaUri({'scheme': 'file', 'path': os.path.expanduser('~')})
+
 
 class MenuItem(object):
 
@@ -98,33 +106,30 @@ class DirectoryConfigureController(PreviewListController):
         self.selector.set_body(body)
 
         self.nodes.set_selector(self.selector)
-    
+
     def node_renderer(self, item, widget):
         super(DirectoryConfigureController,self).node_renderer(item, widget)
         widget.label.label = item.label
         self.frontend.load_from_theme(item.icon, widget.icon)
-
-        if item.uri.path in application.media_directories.directories[item.media_type]:
-            lft = self.frontend.load_from_theme
-            lft('elisa.plugins.poblesec.directory_settings_added', 
-                                                        widget.action_image)
+        path = item.uri.path
+        if path in application.media_directories.directories[item.media_type]:
+            resource = 'elisa.plugins.poblesec.directory_settings_added'
         else:
-            lft = self.frontend.load_from_theme
-            lft('elisa.plugins.poblesec.directory_settings_add', 
-                                                        widget.action_image)
+            resource = 'elisa.plugins.poblesec.directory_settings_add'
+        self.frontend.load_from_theme(resource, widget.action_image)
 
     def handle_input(self, manager, input_event):
         if  self.nodes.focus:
             if input_event.value == EventValue.KEY_GO_LEFT:
-                if self.nodes.action_selected == True:
+                if self.nodes.action_selected:
                     self.nodes.select_list_item()
                     return True
             elif input_event.value == EventValue.KEY_GO_RIGHT:
-                if self.nodes.action_selected == False:
+                if not self.nodes.action_selected:
                     self.nodes.select_action_item()
                     return True
             elif input_event.value == EventValue.KEY_OK:
-                if self.nodes.action_selected == True:
+                if self.nodes.action_selected:
                     index = self.nodes.selected_item_index
                     item = self.model[index]
                     self.toggle_for_model(item, index)
@@ -145,6 +150,9 @@ class DirectoryConfigureController(PreviewListController):
             index = self.model.index(model)
         self.model[index] = model
 
+        # FIXME: Refresh the toggled state of the folder in the parent
+        # controller (see https://bugs.launchpad.net/elisa/+bug/244314).
+
 
 class MediaDirectoryList(SectionMenuController):
 
@@ -175,8 +183,12 @@ class MediaDirectoryList(SectionMenuController):
                            'pictures': 'image'}
 
             file_media_type = translation[media_type]
-            link.controller_args = { 'uri': uri, 'media_type' : file_media_type }
-            link.label = uri.filename
+            link.controller_args = {'uri': uri, 'media_type': file_media_type}
+            if uri.path == HOME_URI.path:
+                # Home directory
+                link.label = _('Home')
+            else:
+                link.label = uri.label
             link.icon = "elisa.plugins.poblesec.%s_folders" % media_type
             self.model.append(link)
         
@@ -198,15 +210,51 @@ class DirectorySettingsController(FilesystemPreviewListController):
     hide_files = True
     node_widget = MenuItemWidgetWithAction
     list_widget = ActionListSelector
-    default_uri = MediaUri("file://"+os.path.expanduser("~"))
     media_type = None
 
-    def initialize(self, uri=default_uri, media_type=media_type):
+    def initialize(self, uri=None, media_type=media_type):
         dfr = super(DirectorySettingsController, self).initialize(uri)
         application.media_directories.refresh_directory_list(media_type)
         self.media_type = media_type
+
+        if uri is None:
+            def got_devices(devices):
+                volumes = [device for device in devices.devices \
+                           if isinstance(device, VolumeModel) \
+                           and device.mount_point is not None]
+                self.model.extend(volumes)
+
+                application.bus.register(self._device_added_cb, NewDeviceDetected)
+                application.bus.register(self._device_removed_cb, DeviceRemoved)
+
+                return self
+
+            def load_items(controller):
+                home = DirectoryModel()
+                home.name = _('Home')
+                home.uri = HOME_URI
+                controller.model.append(home)
+
+                devices_uri = MediaUri('volumes://localhost/?filter=file')
+                devices, dfr_devices = application.resource_manager.get(devices_uri)
+                dfr_devices.addCallback(got_devices)
+                return dfr_devices
+
+            dfr.addCallback(load_items)
+
         return dfr
 
+    def _device_added_cb(self, message, sender):
+        volume = message.model
+        if volume.protocol == 'file' and volume.mount_point is not None:
+            self.model.append(volume)
+
+    def _device_removed_cb(self, message, sender):
+        for volume in self.model:
+            if isinstance(volume, VolumeModel) and volume.udi == message.udi:
+                self.model.remove(volume)
+                break
+
     def nodes_setup(self):
         super(DirectorySettingsController,self).nodes_setup()
         #FIXME: why the renderer and the list are defined with class members
@@ -234,14 +282,20 @@ class DirectorySettingsController(FilesystemPreviewListController):
         super(DirectorySettingsController,self).node_renderer(item, widget)
 
         application.media_directories.refresh_directory_list(self.media_type)
-        if item.uri.path in application.media_directories.all_directories:
-            lft = self.frontend.load_from_theme
-            lft('elisa.plugins.poblesec.directory_settings_added', 
-                                                        widget.action_image)
+
+        if isinstance(item, VolumeModel):
+            # This is necessary on Windows to convert the escaped backslashes
+            # to forward slashes in the mount point.
+            uri = MediaUri({'scheme': 'file', 'path': item.mount_point})
+            path = uri.path
         else:
-            lft = self.frontend.load_from_theme
-            lft('elisa.plugins.poblesec.directory_settings_add', 
-                                                        widget.action_image)
+            path = item.uri.path
+
+        if path in application.media_directories.all_directories:
+            resource = 'elisa.plugins.poblesec.directory_settings_added'
+        else:
+            resource = 'elisa.plugins.poblesec.directory_settings_add'
+        self.frontend.load_from_theme(resource, widget.action_image)
 
     def node_clicked(self, widget, item):
         widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
@@ -252,7 +306,10 @@ class DirectorySettingsController(FilesystemPreviewListController):
 
             self._previous_clicked = selected_item
 
-            file_uri = item.uri
+            if isinstance(item, VolumeModel):
+                file_uri = MediaUri({'scheme': 'file', 'path': item.mount_point})
+            else:
+                file_uri = item.uri
 
             # we assume there was only one controller created with that path
             controllers = self.frontend.retrieve_controllers('/poblesec/browser')
@@ -267,13 +324,17 @@ class DirectorySettingsController(FilesystemPreviewListController):
     def configure_directory(self, model, index=-1):
         if index == -1:
             index = self.model.index(model)
-        if self.media_type:
-            application.media_directories.toggle_directory_installation \
-                                                    (model.uri, self.media_type)
-            self.model[index] = model
+
+        if isinstance(model, VolumeModel):
+            file_uri = MediaUri({'scheme': 'file', 'path': model.mount_point})
         else:
             file_uri = model.uri
 
+        if self.media_type:
+            application.media_directories.toggle_directory_installation \
+                                                    (file_uri, self.media_type)
+            self.model[index] = model
+        else:
             # we assume there was only one controller created with that path
             controllers = self.frontend.retrieve_controllers('/poblesec/browser')
             browser = controllers[0]
@@ -283,15 +344,15 @@ class DirectorySettingsController(FilesystemPreviewListController):
     def handle_input(self, manager, input_event):
         if  self.nodes.focus:
             if input_event.value == EventValue.KEY_GO_LEFT:
-                if self.nodes.action_selected == True:
+                if self.nodes.action_selected:
                     self.nodes.select_list_item()
                     return True
             elif input_event.value == EventValue.KEY_GO_RIGHT:
-                if self.nodes.action_selected == False:
+                if not self.nodes.action_selected:
                     self.nodes.select_action_item()
                     return True
             elif input_event.value == EventValue.KEY_OK:
-                if self.nodes.action_selected == True:
+                if self.nodes.action_selected:
                     index = self.nodes.selected_item_index
                     model = self.nodes.model[index]
                     self.configure_directory(model, index)
