@@ -37,7 +37,7 @@ from elisa.plugins.poblesec.base.coverflow import ImageWithReflectionCoverflowCo
 from elisa.plugins.poblesec.base.grid import GridItemGridController
 from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
 
-from twisted.internet import defer
+from twisted.internet import defer, task
 
 
 _ = install_translation('poblesec')
@@ -57,40 +57,37 @@ class FilesystemController(HierarchyController):
         self.media_type = media_type
         self.gone_back = False
 
-        def resource_loaded(resource):
-            def is_not_hidden(file_model):
-                return not file_model.name.startswith('.')
-
-            non_hidden_files = filter(is_not_hidden, resource.files)
-            non_hidden_files.sort(cmp=lambda x, y: cmp(x.name.lower(), y.name.lower()))
-
-            if self.hide_files == True:
-                def is_directory(file_model):
-                    return isinstance(file_model, DirectoryModel)
-
-                non_hidden_files = filter(is_directory, non_hidden_files)
-
-            for my_file_model in non_hidden_files:
-                try:
-                    media_types = typefinding.typefind(MediaUri(my_file_model.uri))
-                except typefinding.UnknownFileExtension, e:
-                    media_types = []
-
-                if self.media_type in media_types:
-                    my_file_model.media_type = self.media_type
-                else:
+        def find_files(resource):
+            models = sorted(resource.files,
+                    key=lambda model: model.name.lower())
+            for model in models:
+                if model.name.startswith('.'):
+                    yield None
                     continue
 
-            if self.media_type:
-                def is_controller_mediatype(file_model):
-                    if isinstance(file_model, DirectoryModel):
-                        return True
-                    return file_model.media_type == self.media_type
+                if isinstance(model, DirectoryModel):
+                    self.model.append(model)
+                elif not self.hide_files:
+                    
+                    try:
+                        media_types = typefinding.typefind(MediaUri(model.uri))
+                    except typefinding.UnknownFileExtension, e:
+                        media_types = []
 
-                non_hidden_files = filter(is_controller_mediatype, non_hidden_files)
+                    if self.media_type not in media_types:
+                        yield None
+                        continue
 
-            self.model.extend(non_hidden_files)
-            return self
+                    model.media_type = self.media_type
+                    self.model.append(model)
+
+                yield None
+
+        def resource_loaded(resource):
+            dfr = task.coiterate(find_files(resource))
+            dfr.addCallback(lambda result: self)
+
+            return dfr
 
         def got_error(failure, resource):
             self._load_error = failure
