@@ -43,11 +43,9 @@ class History(gobject.GObject, Loggable):
         self.current = None
         self.index = -1
         self._controllers = []
-        self._creation_dfr = None
 
     def append_controller(self, path, display_name, **kwargs):
-        def _append_controller(controller):
-            self._creation_dfr = None
+        def append_controller(controller):
             controller.display_name = display_name
             self._controllers.append(controller)
             previous = self.current
@@ -56,7 +54,7 @@ class History(gobject.GObject, Loggable):
             
             return controller
         
-        def _emit_push_controller(controller):
+        def emit_push_controller(controller):
             index = self._controllers.index(self.current)
             if index > 0:
                 previous = self._controllers[index-1]
@@ -66,37 +64,25 @@ class History(gobject.GObject, Loggable):
             
             return controller
 
-        def _log_controller_creation_error(error, controller_path):
-            self._creation_dfr = None
+        def log_controller_creation_error(error, controller_path):
             log_path = common.application.log_failure(error)
             self.warning("Entering controller with path %s failed. Full" \
                          " failure log at %s" % (controller_path, log_path))
             return error
-
-        # cancel any creation of controller if there was one
-        if self._creation_dfr != None and not self._creation_dfr.called:
-            self._creation_dfr.cancel()
-
-        self._creation_dfr = defer.Deferred()
-        self._creation_dfr.addCallback(_append_controller)
-        self._creation_dfr.addCallback(_emit_push_controller)
-        self._creation_dfr.addErrback(_log_controller_creation_error, path)
-
+        
         try:
             dfr = self.frontend.create_controller(path, **kwargs)
         except:
             error = failure.Failure()
             dfr = defer.fail(error)
+        
+        dfr.addCallback(append_controller)
+        dfr.addCallback(emit_push_controller)
+        dfr.addErrback(log_controller_creation_error, path)
 
-        dfr.chainDeferred(self._creation_dfr)
-
-        return self._creation_dfr
+        return dfr
 
     def go_back(self):
-        # cancel any creation of controller if there was one
-        if self._creation_dfr != None and not self._creation_dfr.called:
-            self._creation_dfr.cancel()
-
         if not self.current:
             self.warning("No current controller in this history.")
             return
@@ -121,3 +107,8 @@ class History(gobject.GObject, Loggable):
             # call clean method of the controller before losing last reference
             # to it
             previous.clean()
+
+    def clear(self):
+        while self.index > 0:
+            self.go_back()
+
