@@ -23,4 +23,9 @@ class Link(object):
         self.label = label
         self.icon = icon
 
+class DeviceLink(Link):
 
+    def __init__(self, udi, *args, **kwargs):
+        super(DeviceLink, self).__init__(*args, **kwargs)
+        self.udi = udi
+        
