@@ -15,6 +15,7 @@
 # for details on that license.
 
 from elisa.plugins.pigment.pigment_controller import PigmentController
+from elisa.plugins.pigment.pigment_frontend import ControllerNotFound
 from elisa.plugins.poblesec.transitions import Transition, FadeIn, FadeOut
 
 from elisa.core.media_uri import MediaUri
@@ -140,6 +141,28 @@ class PoblesecController(PigmentController):
         dfr.addCallback(video_player_created)
         # FIXME: losing a deferred here
 
+        def dvd_player_created(controller):
+            self.dvd_player = controller
+            self.current_player = controller
+            self.widget.add(self.dvd_player.widget)
+            self.dvd_player.widget.width = 1.0
+            self.dvd_player.widget.height = 1.0
+            self.dvd_player.widget.x = 0.0
+            self.dvd_player.widget.y = 0.0
+            self.dvd_player.widget.visible = True
+
+
+        # We don't want the dvd plugin to be a hard dependency for poblesec.
+        try:
+            dfr = self.frontend.create_controller('/poblesec/dvd_player')
+            dfr.addCallback(dvd_player_created)
+            # FIXME: losing a deferred here
+        except ControllerNotFound:
+            self.debug("Not loading /poblesec/dvd_player because its " +
+                       "controller could not be found")
+            self.dvd_player = None
+
+
         def music_player_created(controller):
             self.music_player = controller
             self.widget.add(self.music_player.widget)
@@ -184,10 +207,35 @@ class PoblesecController(PigmentController):
         self.music_player.player.stop()
         FadeOut().apply(self.music_player)
 
+        # stop dvd player
+        if self.dvd_player:
+            self.dvd_player.player.stop()
+            FadeOut().apply(self.dvd_player)
+
         # show video player
         self.current_player = self.video_player
         self.show_current_player()
 
+    def show_dvd_player(self):
+        if not self.dvd_player:
+            return
+
+        # stop slideshow player
+        self.slideshow_player.player.stop()
+        FadeOut().apply(self.slideshow_player)
+
+        # stop music player
+        self.music_player.player.stop()
+        FadeOut().apply(self.music_player)
+
+        # stop video player
+        self.video_player.player.stop()
+        FadeOut().apply(self.video_player)
+
+        # show dvd player
+        self.current_player = self.dvd_player
+        self.show_current_player()
+
     def show_music_player(self):
         # hide slideshow player
         FadeOut().apply(self.slideshow_player)
@@ -196,6 +244,11 @@ class PoblesecController(PigmentController):
         self.video_player.player.stop()
         FadeOut().apply(self.video_player)
 
+        # stop dvd player
+        if self.dvd_player:
+            self.dvd_player.player.stop()
+            FadeOut().apply(self.dvd_player)
+
         # show music player
         self.current_player = self.music_player
         self.show_current_player()
@@ -208,6 +261,11 @@ class PoblesecController(PigmentController):
         self.video_player.player.stop()
         FadeOut().apply(self.video_player)
 
+        # stop dvd player
+        if self.dvd_player:
+            self.dvd_player.player.stop()
+            FadeOut().apply(self.dvd_player)
+
         # show slideshow player
         self.current_player = self.slideshow_player
         self.show_current_player()
@@ -264,6 +322,12 @@ class PoblesecController(PigmentController):
             return controller
 
         return self._background_out.apply(self.popup_controller).addCallback(faded)
+    
+    def stop_all_players(self):
+        self.slideshow_player.player.stop()
+        self.music_player.player.stop()
+        self.video_player.player.stop()
+        self.dvd_player.player.stop()
 
     def handle_input(self, manager, input_event):
 
