@@ -15,22 +15,37 @@
 # for details on that license.
 
 from elisa.core.media_uri import MediaUri
-from elisa.plugins.poblesec.link import Link
-from twisted.internet import defer
 from elisa.core.utils.i18n import install_translation
 from elisa.core.common import application
 
+from elisa.plugins.poblesec.link import Link
+
+from twisted.internet import defer
+
+
 _ = install_translation('poblesec')
 
+
+def search_decorator(controller):
+    link = Link()
+    media_type = 'music'
+    link.controller_path = '/poblesec/search/%s' % media_type
+    link.controller_args = {'mediatype': media_type}
+    link.label = _('Search')
+    link.icon = 'elisa.plugins.poblesec.search'
+    controller.model.append(link)
+    return defer.succeed(None)
+
+
 def music_decorator(controller):
     link = Link()
     link.controller_path = "/poblesec/music_menu"
     link.label = _("Music")
     link.icon = "elisa.plugins.poblesec.music_section"
     controller.model.append(link)
-
     return defer.succeed(None)
 
+
 def albums_decorator(controller):
     link = Link()
     link.controller_path = "/poblesec/music/albums"
@@ -38,9 +53,9 @@ def albums_decorator(controller):
     link.icon = "elisa.plugins.poblesec.album"
     # FIXME: temporarily deactivated
 #    controller.model.append(link)
-
     return defer.succeed(None)
 
+
 def artists_decorator(controller):
     link = Link()
     link.controller_path = "/poblesec/music/artists"
@@ -48,9 +63,9 @@ def artists_decorator(controller):
     link.icon = "elisa.plugins.poblesec.artist"
     # FIXME: temporarily deactivated
 #    controller.model.append(link)
-
     return defer.succeed(None)
 
+
 def filesystem_decorator(controller):
     link = Link()
     link.controller_path = "/poblesec/music/directory_list"
@@ -58,18 +73,18 @@ def filesystem_decorator(controller):
     link.icon = "elisa.plugins.poblesec.music_folders"
     link.controller_args = {'media_type': 'music'}
     controller.model.append(link)
-
     return defer.succeed(None)
 
+
 def internet_decorator(controller):
     link = Link()
     link.controller_path = "/poblesec/music/internet"
     link.label = _("Internet")
     link.icon = "elisa.plugins.poblesec.internet"
     controller.model.append(link)
-
     return defer.succeed(None)
 
+
 def network_decorator(controller):
     link = Link()
     link.controller_path = "/poblesec/music/network"
@@ -77,6 +92,4 @@ def network_decorator(controller):
     link.icon = "elisa.plugins.poblesec.network"
     link.controller_args = {'media_type': 'music'}
     controller.model.append(link)
-
     return defer.succeed(None)
-
