@@ -252,11 +252,14 @@ class AlbumsViewMode(GenericListViewMode):
         return resource
 
     def get_image(self, item, theme):
+        def got_artist(artist):
+            return album_cover_retriever.get_cover_uri(item, artist)
+
         def got_image_uri(result_model):
             try:
                 if result_model.cover_uri is None:
                     return None
-                return _get_thumbnail(result_model.cover_uri)                
+                return _get_thumbnail(result_model.cover_uri)
             except AttributeError:
                 return None
 
@@ -266,9 +269,8 @@ class AlbumsViewMode(GenericListViewMode):
         if hasattr(item, 'cover_uri') and item.cover_uri is not None:
             image_deferred = defer.succeed(item)
         else:
-            artist = None
-            # FIXME: retrieve the artist
-            image_deferred = album_cover_retriever.get_cover_uri(item, artist)
+            image_deferred = item.get_artist_name()
+            image_deferred.addCallback(got_artist)
         image_deferred.addCallbacks(got_image_uri, _failure)
         return image_deferred
 
@@ -302,12 +304,13 @@ class TracksViewMode(GenericListViewMode):
         return resource
 
     def get_image(self, item, theme):
+
         def got_image_uri(result_model):
             try:
                 if result_model.cover_uri is None:
                     return None
                 item.cover_uri = result_model.cover_uri
-                return _get_thumbnail(result_model.cover_uri)                
+                return _get_thumbnail(result_model.cover_uri)
             except AttributeError:
                 return None
 
@@ -319,16 +322,18 @@ class TracksViewMode(GenericListViewMode):
             image_deferred.addCallbacks(got_image_uri, _failure)
             return image_deferred
         else:
-            artist = None
-            # FIXME: retrieve the artist
+            def got_artist(artist, album):
+                return album_cover_retriever.get_cover_uri(album, artist)
 
             def got_album(album):
                 if hasattr(album, 'cover_uri') and album.cover_uri is not None:
                     return defer.succeed(album)
                 elif album:
-                    return album_cover_retriever.get_cover_uri(album, artist)
+                    dfr = album.get_artist_name()
+                    dfr.addCallback(got_artist, album)
+                    return dfr
 
-            album_deferred = item.album
+            album_deferred = item.get_album()
             album_deferred.addCallback(got_album)
             album_deferred.addCallbacks(got_image_uri, _failure)
             return album_deferred
