@@ -30,14 +30,20 @@ from elisa.plugins.pigment.widgets import notifying_list
 from elisa.plugins.pigment.widgets.list_horizontal import ListHorizontal
 from elisa.plugins.poblesec.player_video import PlayerOsd as VideoPlayerOsd
 from elisa.plugins.poblesec.player_video import PlayerOsdStatusMixin
-from elisa.plugins.poblesec.long_loading_image import LongLoadingImage
+from elisa.plugins.poblesec.widgets.long_loading_image import LongLoadingImage
+
+from elisa.core import common
 
 from twisted.internet import reactor
-import gobject
-import os, os.path, md5
+import gobject, os, os.path, md5, time
 
 from pgm.utils import maths
 
+try:
+    from elisa.plugins.database.models import File as DBFile
+except ImportError:
+    # database missing
+    DBFile = None
 
 class SlideshowPlayerOsdStatus(PlayerOsdStatusMixin):
     factor = 1
@@ -144,8 +150,11 @@ class Slideshow(gobject.GObject, Loggable):
         self.background = ListFading()
 
         def on_selected_item_changed(background, item, prev_item):
-            index = self.background.model.index(item)
-            self._set_current_index(index)
+            if item:
+                # item is None when the new selected index is -1 (which happens
+                # when the model is empty)
+                index = self.background.model.index(item)
+                self._set_current_index(index)
 
         self.background.connect('selected-item-changed',
                                 on_selected_item_changed)
@@ -167,7 +176,9 @@ class Slideshow(gobject.GObject, Loggable):
                 dfr = getattr(widget, name)
                 if dfr != None and not dfr.called:
                     # Swallow the error raised by the cancellation
-                    dfr.addErrback(lambda x: None)
+                    def _failure(failure):
+                        failure.trap(cancellable_defer.CancelledError)
+                    dfr.addErrback(_failure)
                     dfr.cancel()
                 delattr(widget, name)
 
@@ -217,6 +228,28 @@ class Slideshow(gobject.GObject, Loggable):
         self.debug("retrieving full resolution image from %s" % uri)
         return self._retrieve_and_cache(uri)
 
+    def _update_playcount_and_time(self, uri):
+        if uri.scheme != 'file':
+            # database only support files
+            return
+
+        def update(result):
+            if not result:
+                return
+
+            self.debug("updating playcount")
+            result.playcount += 1
+            result.last_played = time.time()
+
+        store = common.application.store
+
+        if not store or not DBFile:
+            return
+
+        dfr = store.get(DBFile, uri.path)
+        dfr.addCallback(update)
+        return dfr
+
     def _retrieve_and_cache(self, uri):
         path = self._compute_cached_path(uri)
 
@@ -302,6 +335,7 @@ class Slideshow(gobject.GObject, Loggable):
 
     def _set_current_index(self, index):
         previous_index = self.current_index
+        self._update_playcount_and_time(self.playlist[index].references[-1])
         self.current_index = index
         self.emit('current-index-changed', index, previous_index)
 
