@@ -41,7 +41,7 @@ from elisa.plugins.base.models.media import PlayableModel
 from elisa.core.log import Loggable
 import gobject
 
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 from pgm.timing import implicit
 import pgm, time
 import gst
@@ -505,6 +505,8 @@ class Player(gobject.GObject, Loggable):
         pbus.connect('message::element', self._message_element_cb)
         pbus.add_signal_watch()
 
+        self._play_deferreds = []
+
     def _message_eos_cb(self, bus, message):
         self.pipeline.set_state(gst.STATE_NULL)
         self.play_next()
@@ -623,27 +625,45 @@ class Player(gobject.GObject, Loggable):
             if self.status != self.PLAYING:
                 self.play()
         else:
+            for dfr in self._play_deferreds:
+                if hasattr(dfr, 'cancel'):
+                    dfr.cancel()
+
+            def remove_from_play_deferreds(result, deferred):
+                self._play_deferreds.remove(deferred)
+                return result
+
             model = self.playlist[index]
-            self.filename = model.title or model.uri.filename
-            self.stop()
-            self.pipeline.set_property('uri', model.uri)
-            self._load_subs(model.uri)
+            if not isinstance(model, PlayableModel):
+                dfr = model.get_playable_model()
+                self._play_deferreds.append(dfr)
+                dfr.addBoth(remove_from_play_deferreds, dfr)
+            else:
+                dfr = defer.succeed(model)
 
-            # FIXME: make this configurable
-            vis_plugin_name = 'libvisual_jess'
-            if platform.system() == 'Windows':
-                vis_plugin_name = 'goom2k1'
+            dfr.addCallback(self._play_playable_model_at_index, index)
 
-            try:
-                visu = ElisaVisualisationBin(vis_plugin_name, 400,
-                                             self.image)
-                self.pipeline.set_property('vis-plugin', visu)
-            except gst.ElementNotFoundError:
-                pass
+    def _play_playable_model_at_index(self, model, index):
+        self.filename = model.title or model.uri.filename
+        self.stop()
+        self.pipeline.set_property('uri', model.uri)
+        self._load_subs(model.uri)
 
-            self.play()
-            self._update_playcount_and_time(model)
-            self.current_index = index
+        # FIXME: make this configurable
+        vis_plugin_name = 'libvisual_jess'
+        if platform.system() == 'Windows':
+            vis_plugin_name = 'goom2k1'
+
+        try:
+            visu = ElisaVisualisationBin(vis_plugin_name, 400,
+                                         self.image)
+            self.pipeline.set_property('vis-plugin', visu)
+        except gst.ElementNotFoundError:
+            pass
+
+        self.play()
+        self._update_playcount_and_time(model)
+        self.current_index = index
 
     def stop(self):
         self.pipeline.set_state(gst.STATE_NULL)
