@@ -602,9 +602,7 @@ class PoblesecBrowserController(BrowserController):
         elif index == bb.STOP:
             controllers = self.frontend.retrieve_controllers('/poblesec')
             main = controllers[0]
-            main.slideshow_player.player.stop()
-            main.music_player.player.stop()
-            main.video_player.player.stop()
+            main.stop_all_players()
         elif index == bb.LIST:
             # we assume current is a ListSwitcher
             switcher = self.history.current
