@@ -107,7 +107,7 @@ class PopupWidget(Widget):
         self.callbacks = {}
 
         def button_clicked_cb(text, x, y, z, button, time, pressure):
-            self.callbacks[text.label]()
+            self.callbacks[text]()
             return True
 
         for text, callback in self.buttons:
@@ -124,7 +124,7 @@ class PopupWidget(Widget):
             button.bg_color = (0, 0, 0, 0)
             button.visible = True
 
-            self.callbacks[text] = callback
+            self.callbacks[button] = callback
             button.connect('clicked', button_clicked_cb)
 
             self.button_widgets.append(button)
