@@ -21,11 +21,12 @@ from elisa.plugins.poblesec.base.hierarchy import HierarchyController
 from elisa.plugins.poblesec.base.list import GenericListViewMode
 from elisa.plugins.poblesec.base.preview_list import \
     MenuItemPreviewListController
-from elisa.plugins.poblesec.link import Link
+from elisa.plugins.poblesec.link import Link, DeviceLink
 
 from elisa.plugins.base.messages.device import NewDeviceDetected, \
                                                NewUnknownDevice, \
                                                DeviceRemoved
+from elisa.plugins.base.models.device import VolumeModel
 
 from twisted.internet import defer
 
@@ -68,6 +69,23 @@ class SectionMenuViewMode(GenericListViewMode):
 class SectionMenuController(SectionMenuHierarchy, MenuItemPreviewListController):
     view_mode = SectionMenuViewMode
 
+# We implement this as a decorator to be able to ensure removable devices
+# appear at the end.
+def hardware_decorator(controller):
+    uri = MediaUri("volumes://?filter=removable")
+    model, dfr = common.application.resource_manager.get(uri, None)
+
+    def do_list_hardware(model):
+        devices = sorted(model.devices, key=lambda i: i.name.lower())
+
+        for dev in devices:
+            if dev.protocol == 'file' and dev.mount_point != None:
+               controller._add_link_from_device_model(dev) 
+
+
+    dfr.addCallback(do_list_hardware)
+
+    return dfr
 
 class HardwareListControllerMixin(SectionMenuController):
 
@@ -85,38 +103,34 @@ class HardwareListControllerMixin(SectionMenuController):
         return dfr
 
     def _device_add_cb(self, msg, sender):
-        # FIXME: only file protocol supported at this time, no audio cd's nor dvd's
-        link = Link()
-
-        if msg.model.protocol == 'file':
-            file_media_type = self.translation[self.media_type]
-            link.controller_path = "/poblesec/%s/filesystem" % (self.media_type)
-            link.controller_args = {'uri': msg.model.mount_point, 'media_type': file_media_type}
-            link.icon = "elisa.plugins.poblesec.removable_device"
-        elif msg.model.protocol == 'ipod' and self.media_type == 'music':
-            link.controller_path = "/poblesec/music/ipod"
-            link.controller_args = {'uri': msg.model.mount_point}
-            link.icon = "elisa.plugins.poblesec.ipod"
-        else:
-            # not supported yet
-            return
-
-        for existing_link in self.model:
-            uri = existing_link.controller_args.get('uri', None)
-            if msg.model.mount_point and uri == msg.model.mount_point:
-                # already added, can be in some cases ...
-                return
-
-        link.label = msg.model.label
-        self.model.append(link)
+        if msg.model.protocol == 'file' and msg.model.mount_point != None:
+            self._add_link_from_device_model(msg.model)
 
     def _device_remove_cb(self, msg, sender):
         for link in self.model:
-            uri = link.controller_args.get('uri', None)
-            if None not in (uri, msg.udi) and msg.udi == uri:
+            if isinstance(link, DeviceLink) and link.udi == msg.udi:
                 self.model.remove(link)
                 return
 
+    def _add_link_from_device_model(self, model):
+        link = DeviceLink(model.udi)
+        path = ''
+        if isinstance(model, VolumeModel):
+            path = model.mount_point
+        else:
+            path = model.device
+
+        uri = MediaUri({'scheme': model.protocol,
+                        'host': '',
+                        'path': path})
+        file_media_type = self.translation[self.media_type]
+        link.controller_path = "/poblesec/%s/filesystem" % (self.media_type)
+        link.controller_args = {'uri': uri, 'media_type': file_media_type}
+        link.icon = "elisa.plugins.poblesec.removable_device"
+
+        link.label = model.name
+        self.model.append(link)
+
     def clean(self):
         dfr = super(HardwareListControllerMixin, self).clean()
         bus = common.application.bus
